@@ -1,13 +1,8 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
  * ipmonitor.c		"ip monitor".
  *
- *		This program is free software; you can redistribute it and/or
- *		modify it under the terms of the GNU General Public License
- *		as published by the Free Software Foundation; either version
- *		2 of the License, or (at your option) any later version.
- *
  * Authors:	Alexey Kuznetsov, <kuznet@ms2.inr.ac.ru>
- *
  */
 
 #include <stdio.h>
@@ -27,28 +22,33 @@
 static void usage(void) __attribute__((noreturn));
 static int prefix_banner;
 int listen_all_nsid;
+struct rtnl_ctrl_data *ctrl_data;
+int do_monitor;
 
 static void usage(void)
 {
 	fprintf(stderr,
 		"Usage: ip monitor [ all | OBJECTS ] [ FILE ] [ label ] [ all-nsid ]\n"
 		"                  [ dev DEVICE ]\n"
-		"OBJECTS :=  address | link | mroute | neigh | netconf |\n"
-		"            nexthop | nsid | prefix | route | rule | stats\n"
+		"OBJECTS :=  address | link | mroute | maddress | acaddress | neigh |\n"
+		"            netconf | nexthop | nsid | prefix | route | rule | stats\n"
 		"FILE := file FILENAME\n");
 	exit(-1);
 }
 
-static void print_headers(FILE *fp, char *label, struct rtnl_ctrl_data *ctrl)
+void print_headers(FILE *fp, const char *label)
 {
+	if (!do_monitor)
+		return;
+
 	if (timestamp)
 		print_timestamp(fp);
 
 	if (listen_all_nsid) {
-		if (ctrl == NULL || ctrl->nsid < 0)
+		if (ctrl_data == NULL || ctrl_data->nsid < 0)
 			fprintf(fp, "[nsid current]");
 		else
-			fprintf(fp, "[nsid %d]", ctrl->nsid);
+			fprintf(fp, "[nsid %d]", ctrl_data->nsid);
 	}
 
 	if (prefix_banner)
@@ -60,6 +60,8 @@ static int accept_msg(struct rtnl_ctrl_data *ctrl,
 {
 	FILE *fp = (FILE *)arg;
 
+	ctrl_data = ctrl;
+
 	switch (n->nlmsg_type) {
 	case RTM_NEWROUTE:
 	case RTM_DELROUTE: {
@@ -76,11 +78,9 @@ static int accept_msg(struct rtnl_ctrl_data *ctrl,
 
 		if (r->rtm_family == RTNL_FAMILY_IPMR ||
 		    r->rtm_family == RTNL_FAMILY_IP6MR) {
-			print_headers(fp, "[MROUTE]", ctrl);
 			print_mroute(n, arg);
 			return 0;
 		} else {
-			print_headers(fp, "[ROUTE]", ctrl);
 			print_route(n, arg);
 			return 0;
 		}
@@ -88,32 +88,27 @@ static int accept_msg(struct rtnl_ctrl_data *ctrl,
 
 	case RTM_NEWNEXTHOP:
 	case RTM_DELNEXTHOP:
-		print_headers(fp, "[NEXTHOP]", ctrl);
 		print_cache_nexthop(n, arg, true);
 		return 0;
 
 	case RTM_NEWNEXTHOPBUCKET:
 	case RTM_DELNEXTHOPBUCKET:
-		print_headers(fp, "[NEXTHOPBUCKET]", ctrl);
 		print_nexthop_bucket(n, arg);
 		return 0;
 
 	case RTM_NEWLINK:
 	case RTM_DELLINK:
 		ll_remember_index(n, NULL);
-		print_headers(fp, "[LINK]", ctrl);
 		print_linkinfo(n, arg);
 		return 0;
 
 	case RTM_NEWADDR:
 	case RTM_DELADDR:
-		print_headers(fp, "[ADDR]", ctrl);
 		print_addrinfo(n, arg);
 		return 0;
 
 	case RTM_NEWADDRLABEL:
 	case RTM_DELADDRLABEL:
-		print_headers(fp, "[ADDRLABEL]", ctrl);
 		print_addrlabel(n, arg);
 		return 0;
 
@@ -127,18 +122,15 @@ static int accept_msg(struct rtnl_ctrl_data *ctrl,
 				return 0;
 		}
 
-		print_headers(fp, "[NEIGH]", ctrl);
 		print_neigh(n, arg);
 		return 0;
 
 	case RTM_NEWPREFIX:
-		print_headers(fp, "[PREFIX]", ctrl);
 		print_prefix(n, arg);
 		return 0;
 
 	case RTM_NEWRULE:
 	case RTM_DELRULE:
-		print_headers(fp, "[RULE]", ctrl);
 		print_rule(n, arg);
 		return 0;
 
@@ -148,21 +140,25 @@ static int accept_msg(struct rtnl_ctrl_data *ctrl,
 
 	case RTM_NEWNETCONF:
 	case RTM_DELNETCONF:
-		print_headers(fp, "[NETCONF]", ctrl);
 		print_netconf(ctrl, n, arg);
 		return 0;
 
 	case RTM_DELNSID:
 	case RTM_NEWNSID:
-		print_headers(fp, "[NSID]", ctrl);
 		print_nsid(n, arg);
 		return 0;
 
 	case RTM_NEWSTATS:
-		print_headers(fp, "[STATS]", ctrl);
 		ipstats_print(n, arg);
 		return 0;
 
+	case RTM_DELMULTICAST:
+	case RTM_NEWMULTICAST:
+	case RTM_DELANYCAST:
+	case RTM_NEWANYCAST:
+		print_addrinfo(n, arg);
+		return 0;
+
 	case NLMSG_ERROR:
 	case NLMSG_NOOP:
 	case NLMSG_DONE:
@@ -189,16 +185,21 @@ static int accept_msg(struct rtnl_ctrl_data *ctrl,
 #define IPMON_LRULE		BIT(8)
 #define IPMON_LNSID		BIT(9)
 #define IPMON_LNEXTHOP		BIT(10)
+#define IPMON_LMADDR		BIT(11)
+#define IPMON_LACADDR		BIT(12)
 
 #define IPMON_L_ALL		(~0)
 
 int do_ipmonitor(int argc, char **argv)
 {
 	unsigned int groups = 0, lmask = 0;
+	/* "needed" mask, failure to enable is an error */
+	unsigned int nmask;
 	char *file = NULL;
 	int ifindex = 0;
 
 	rtnl_close(&rth);
+	do_monitor = 1;
 
 	while (argc > 0) {
 		if (matches(*argv, "file") == 0) {
@@ -210,6 +211,10 @@ int do_ipmonitor(int argc, char **argv)
 			lmask |= IPMON_LLINK;
 		} else if (matches(*argv, "address") == 0) {
 			lmask |= IPMON_LADDR;
+		} else if (matches(*argv, "maddress") == 0) {
+			lmask |= IPMON_LMADDR;
+		} else if (strcmp(*argv, "acaddress") == 0) {
+			lmask |= IPMON_LACADDR;
 		} else if (matches(*argv, "route") == 0) {
 			lmask |= IPMON_LROUTE;
 		} else if (matches(*argv, "mroute") == 0) {
@@ -253,6 +258,7 @@ int do_ipmonitor(int argc, char **argv)
 	ipneigh_reset_filter(ifindex);
 	ipnetconf_reset_filter(ifindex);
 
+	nmask = lmask;
 	if (!lmask)
 		lmask = IPMON_L_ALL;
 
@@ -327,11 +333,36 @@ int do_ipmonitor(int argc, char **argv)
 	}
 
 	if (lmask & IPMON_LSTATS &&
-	    rtnl_add_nl_group(&rth, RTNLGRP_STATS) < 0) {
+	    rtnl_add_nl_group(&rth, RTNLGRP_STATS) < 0 &&
+	    nmask & IPMON_LSTATS) {
 		fprintf(stderr, "Failed to add stats group to list\n");
 		exit(1);
 	}
 
+	if (lmask & IPMON_LMADDR) {
+		if ((!preferred_family || preferred_family == AF_INET) &&
+		    rtnl_add_nl_group(&rth, RTNLGRP_IPV4_MCADDR) < 0) {
+			fprintf(stderr,
+				"Failed to add ipv4 mcaddr group to list\n");
+			exit(1);
+		}
+		if ((!preferred_family || preferred_family == AF_INET6) &&
+		    rtnl_add_nl_group(&rth, RTNLGRP_IPV6_MCADDR) < 0) {
+			fprintf(stderr,
+				"Failed to add ipv6 mcaddr group to list\n");
+			exit(1);
+		}
+	}
+
+	if (lmask & IPMON_LACADDR) {
+		if ((!preferred_family || preferred_family == AF_INET6) &&
+		    rtnl_add_nl_group(&rth, RTNLGRP_IPV6_ACADDR) < 0) {
+			fprintf(stderr,
+				"Failed to add ipv6 acaddr group to list\n");
+			exit(1);
+		}
+	}
+
 	if (listen_all_nsid && rtnl_listen_all_nsid(&rth) < 0)
 		exit(1);
 
