@@ -1,13 +1,8 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
  * tc_stab.c		"tc qdisc ... stab *".
  *
- *		This program is free software; you can redistribute it and/or
- *		modify it under the terms of the GNU General Public License
- *		as published by the Free Software Foundation; either version
- *		2 of the License, or (at your option) any later version.
- *
  * Authors:	Jussi Kivilinna, <jussi.kivilinna@mbnet.fi>
- *
  */
 
 #include <stdio.h>
